;;;; gnu.scm - Display a GNU Head when a user says "I love GNU" -*- scheme -*-
;;;; Copyright (C) 2002, 2003  Neal H. Walfield  <neal@cs.uml.edu>
;;;;
;;;; This file is part of Deride
;;;;
;;;; Deride is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2, or (at your option)
;;;; any later version.
;;;; 
;;;; Deride is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with the Deride; see the file COPYING.  If not, write to
;;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (backend gnu))

(use-modules (comm hooks))
(use-modules (comm send))

(use-modules (conf deride))

(use-modules (utils utils))

(use-modules (ice-9 regex))

;; Map of names to lists of file-ports.  Keys should compare equal
;; using `eqv?'.
(define who-loves-gnu '())

;; Data is an assoc list of loves -> (last-said times-said)

;; Get the data associate with a lover.
(define (get-lover-data lover)
  (assoc-ref who-loves-gnu (string-downcase lover)))

;; Set the data associate with a lover.
(define (set-lover-data lover data)
  (set! who-loves-gnu
	(assoc-set! who-loves-gnu (string-downcase lover) data)))

;; Return the data for the OBJECT that LOVER loves.
(define (get-lover-object lover object)
  (assoc-ref (get-lover-data lover) (string-downcase object)))

;; Set DATA as the data for the OBJECT that LOVER loves.
(define (set-lover-object lover object data)
  (set-lover-data
   lover
   (assoc-set! (or (get-lover-data lover) '())
	       (string-downcase object) data)))

;; Forget that LOVER loves OBJECT.
(define (drop-lover-object lover object)
  (set-lover-data
   lover
   (assoc-remove! (get-lover-data lover) (string-downcase object))))

;; Return the data that a lover with LOVER-DATA loves (LOVER-DATA was
;; likely returned by get-lover-data).
(define (get-object lover-data object)
  (assoc-ref lover-data (string-downcase object)))

(define love-re (make-regexp
		 (string-append
		  "(([a-zA-Z0-9_-]+) *[,:] +)?" ; Target
		  "((what) +)?"                 ; What
		  "((does|do) +)?"	        ; Question
		  "(([a-zA-Z0-9_ -]*[a-zA-Z0-9_-]) +)"       ; Subject
		  "((loves?) *)"                ; love or loves
		  "(([a-zA-Z0-9_ -]*[a-zA-Z0-9_-]) *)?" ; Object
		  "([\?\!\.])?")                ; Punctuation
		 regexp/icase))

(define (does-deride-love object)
  ;; FIXME: This should be called once.
  (set-lover-object deride-nick "gnu" '(0 1))

  (let ((love (or (get-lover-object deride-nick object)
		  '(0 0))))
    (or (and love
	     (cadr love))
	0)))

(add-hook!
 comm-received-forum-message-hook
 (lambda (sender forum say msg)
   ;; Do not parse ourselves.
   (if (not (string-ci=? sender deride-nick))
       (let ((match (regexp-exec love-re msg)))
	 (if match
	     (let*
		 ((target (match:substring match 2))
		  (what (match:substring match 4))
		  (does (match:substring match 6))
		  (subject (match:substring match 8))
		  (love (match:substring match 10))
		  (object (match:substring match 12))
		  (punctuation (or (and does "?")
				   (or (match:substring match 13)
				       ".")))
		  (question (string-ci=? punctuation "?")))

	       (display "target: ") (display target) (newline)
	       (display "what: ") (display what) (newline)
	       (display "does: ") (display does) (newline)
	       (display "subject: ") (display subject) (newline)
	       (display "love: ") (display love) (newline)
	       (display "object: ") (display object) (newline)
	       (display "punctuation: ") (display punctuation) (newline)
	       (display "question: ") (display question) (newline)
	       (display who-loves-gnu) (newline)
	       (cond

		;; What does someone love.
		(what
		 (if (and subject does (not object))
		     (let ((subject (if (and (string-ci=? subject "you")
					     target
					     (string-ci=? target deride-nick))
					deride-nick
					subject)))
		       (let ((data (get-lover-data subject)))
			 (send-to-forum
			  forum
			  (string-append
			   sender ": "
			   (if (not data)
			       "I don't know."
			       (string-append
				subject " loves " 
				(car (random-element-from-list data))))))))))

		;; SENDER has told us that s/he loves something!
		((and (string-ci=? "i" subject) object)
		 (if (string-ci=? punctuation "?")
		     (send-to-forum
		      forum
		      (string-append sender ": You should!" " I do!"))
		     (let ((lover (or (get-lover-object sender object)
				      '(0 0))))
		       ;; Record that SENDER loves OBJECT
		       (set-lover-object
			sender
			object
			(list (current-time)
			      (1+ (or (and lover (cadr lover))
				      0))))
		       ;; Do we love that object too?  Maybe we should
		       ;; respond.
		       (if (< 0 (does-deride-love object))
			   (send-to-forum forum "Me too!")))))

		;; Question/statement about me (or to me about me)
		((and subject object
		      (or (string-ci=? subject deride-nick)
			  (and target
			       (string-ci=? subject "you")
			       (string-ci=? target deride-nick)))
		      (let ((karma (does-deride-love object)))
			(cond
			 ((< 0 karma)
			  (send-to-forum forum "Indeed."))
			 ((> 0 karma)
			  (send-to-forum forum "No way!"))
			 ((= 0 karma)
			  (send-to-forum forum "Not that I am aware of."))))))

		;; Who loves something?
		((and (or (string-ci=? subject "who")
			  (string-ci=? subject "whom"))
		      object)
		 (letrec
		     ((iter
		       (lambda (l r)
			 (display "l: ") (display l) (newline)
			 (display "r: ") (display r) (newline)
			 (if (eq? l '())
			     r
			     (iter
			      (cdr l) 
			      (if (get-object (car l) object)
				  (cons (cond
					 ((string-ci=? (caar l) deride-nick) "I")
					 ((string-ci=? (caar l) sender) "you")
					 (else (caar l)))
					r)
				  r))))))
		   (let ((users (iter who-loves-gnu '())))
		     (display "Who users: ") (display users) (newline)
		     (send-to-forum
		      forum
		      (string-append
		       sender ": "
		       (cond
			((eq? users '()) "No one that I am aware of.")
			((eq? (cdr users) '()) (string-append (car users)
							      " loves "
							      object "."))
			(else (string-append (list-of-strings->string
					      users ", " " and ")
					     " love " object "."))))))))

		;; Question.
		((and question object)
		 (let*
		     ((lover (or (get-lover-object
				  subject
				  object) '(0 0)))
		      (last-love (car lover))
		      (much-love (cadr lover)))

		   (if (and (string-ci=? subject sender)
			    (or (not target)
				(and target (not (string-ci=?
						  subject target)))))
		       ;; Subject is sender (i.e. asking about self).
		       (if (= 0 much-love)
			   (send-to-forum forum
					  (string-append
					   sender ": That is a question "
					   "you need to ask yourself."))
			   (send-to-forum forum
					  (string-append
					   sender ": You have said "
					   "so yourself.")))

		       ;; Sender is asking about someone else.
		       (cond
			((= 0 much-love)
			 (if (or (not target)
				 (string-ci=? target deride-nick))
			     (send-to-forum forum
					    (string-append
					     sender ": I don't know. "
					     "You could ask."))))

			((< 0 much-love)
			 (send-to-forum forum
					(string-append 
					 sender ": Yes, " subject
					 " loves " object "."))))))))))))))


(define love-command-re (make-regexp
			(string-append
			 "^ *([a-zA-Z0-9_-]+) +" ; Subject
			 "(love|hate)s? +"       ; Verb
			 "([a-zA-Z0-9_ -]+) *")  ; Object
			regexp/icase))

(define forget-command-re (make-regexp
			   (string-append
			    " *forget *(that *)?"
			    "([a-zA-Z0-9_-]+) +"  ; Subject
			    "(love|hate)s? +"     ; Verb
			    "([a-zA-Z0-9_ -]+) *") ; Object
			   regexp/icase))

(add-hook! comm-received-personal-message-hook
	   (lambda (sender say msg)
	     (display msg) (newline)
	     (let ((match (regexp-exec love-command-re msg)))
	       (if match
		   (let*
		       ((subject (match:substring match 1))
			(verb (match:substring match 2))
			(object (match:substring match 3)))
		     (display subject) (newline)
		     (display verb) (newline)
		     (display object) (newline)
		     (if (and subject verb object)
			 (begin
			   (if (string-ci=? verb "love")
			       (set-lover-object subject object
						 (list (current-time) 1))
			       (set-lover-object subject object
						 (list (current-time) -1)))
			   (send-to-user sender
					 (string-append "Okay, "
							subject " "
							verb "s "
							object "."))))
		     (display who-loves-gnu) (newline))

		   (let ((match (regexp-exec forget-command-re msg)))
		     (display "Forgetting...") (display msg) (newline)
		     (if match
			 (let*
			     ((subject (match:substring match 2))
			      (verb (match:substring match 3))
			      (object (match:substring match 4)))
			   (display subject) (newline)
			   (display verb) (newline)
			   (display object) (newline)
			   (if (and subject verb object)
			       (if (get-lover-object subject object)
				   (begin
				     (send-to-user sender
						   (string-append
						    "Forgetting that "
						    subject " loves "
						    object "."))
				     (drop-lover-object subject object))
				   (send-to-user sender
						 (string-append
						  subject
						  " does not love "
						  object ".")))))))))))

; 		((or (string-ci=? "who loves gnu?" msg)
; 		     (string-ci=? (string-append deride-nick " loves gnu")
; 
; 		   ;; Image taken:
; 		   ;; from http://www.gnu.org/graphics/gnu-ascii.html
; 		   (send-to-forum forum "  ,           , ")
; 		   (send-to-forum forum " /             \\ ")
; 		   (send-to-forum forum "((__-^^-,-^^-__)) ")
; 		   (send-to-forum forum " `-_---' `---_-' ")
; 		   (send-to-forum forum "  `--|o` 'o|--' ")
; 		   (send-to-forum forum "     \\  `  /")
; 		   (send-to-forum forum "      ): :( ")
; 		   (send-to-forum forum "      :o_o: ")
; 		   (send-to-forum forum "       \"-\" ")
; 		   ())
; 		 '())))
