;;;; deride.scm - Interesting configuration parameters. -*- scheme -*-
;;;; Copyright (C) 2002, 2003  Neal H Walfield  <neal@cs.uml.edu>
;;;;
;;;; This file is part of Deride
;;;;
;;;; Deride is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2, or (at your option)
;;;; any later version.
;;;; 
;;;; Deride is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with the Deride; see the file COPYING.  If not, write to
;;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (conf deride))

(use-modules (backend logger))
(use-modules (backend gnu))
(use-modules (backend uptime))
(use-modules (comm hooks))

;; The used for most of the files that follow.
(define base-directory "/home/deride")

;; The nick name to use
(define-public deride-nick "deride")

;; XXX: Requires deride-nick to be set.
(use-modules (backend lastseen))

;; Our `Real Name'
(define-public deride-real-name "G. Hird Deride")

;; A list of irc servers and ports on which to attempt to connect to them
;; E.g. '(("server1" . (port1 port2)) ("server2" . (port1 port2 port3)))
(define-public irc-servers '(("irc.freenode.org" . (6667))
			     ("80.239.146.2" . (6667))
			     ("128.193.0.30" . (6667))))

(define logger-directory (string-append base-directory "/public_html"))
(define logger-private-directory (string-append base-directory
						"/deride/logs"))

(set! get-forum-log-file
      (lambda (forum old-port)
	(let* ((now (localtime (current-time)))
	       (logger-directory (if (eq? (string-ref forum 0) #\#)
				     logger-directory
				     logger-private-directory)))
	  (open-file (string-append logger-directory "/" forum "-"
				    ;; Year
				    (number->string (+ 1900 (tm:year now)))
				    ;; Month
				    (if (< (tm:mon now) 9) "0" "")
				    (number->string (+ 1 (tm:mon now)))
				    ;; Day
				    (if (< (tm:mday now) 10) "0" "")
				    (number->string (tm:mday now)))
		     "al"))))

(define forum-last-write-time-alist '())

;; Get the last write time associated with a forum name.
(define (get-forum-last-write forum)
  (let ((last-write (assoc forum forum-last-write-time-alist)))
    (if last-write
	(cdr last-write)
	0)))

;; Get the last write time associated with a forum name.
(define (set-forum-last-write forum time)
  (set! forum-last-write-time-alist 
	(assoc-set! forum-last-write-time-alist forum time)))

(add-hook! logger-is-logging-hook
	   (lambda (forum file-port)
	     (let* ((today (inexact->exact
			    (floor (/ (current-time) (* 60 60 24))))))
	       (if (not (= today (get-forum-last-write forum)))
		    (logger-set-forums-file-port
		     forum (get-forum-log-file forum file-port)))
	       (set-forum-last-write forum today))))

;;; The seen database

;; The seen database filename.
(set! seen-db (string-append base-directory "/deride/seen-db"))

;; Frequency with which to flush the database to disk.
;; (in seconds; default: 5 minutes)
; (set! seen-db-flush-frequency (* 5 60))

;; The channel to join on startup.
(define-public irc-channel "#hird")

(define-public (client-display s)
    (run-hook comm-global-status-message-hook s))

