;;;; utils.scm - Generally useful functions. -*- scheme -*-
;;;; Copyright (C) 2003  Neal H Walfield  <neal@cs.uml.edu>
;;;;
;;;; This file is part of Deride
;;;;
;;;; Deride is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2, or (at your option)
;;;; any later version.
;;;; 
;;;; Deride is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with the Deride; see the file COPYING.  If not, write to
;;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (utils utils))

(define-public (alist-length list)
  (letrec ((len (lambda (list, count)
		  (if (eq? list '())
		      count
		      (len (cdr list) (1+ count))))))
    (len list 0)))

;; Return a random element from a list.
(define-public (random-element-from-list list)
  (list-ref list (random (length list))))

;; Turn LIST, a list of strings into a string with the elements
;; seperated by SEP and the last two elements seperated by FINALSEP.
;; E.g. (list-of-strings->string ("a" "b" "c" "d") ", " " and ")
;;       => "a, b, c and d"
(define-public (list-of-strings->string list sep finalsep)
  (letrec ((tailed-conv
	    (lambda (string-so-far list-to-do)
	      (if (eqv? (cdr list-to-do) '())
		  ;; List with a single elment
		  (string-append string-so-far finalsep (car list-to-do))
		  ;; More than one element left to process
		  (tailed-conv
		   (string-append string-so-far sep (car list-to-do))
		   (cdr list-to-do))))))
    (if (eqv? list '())
	""
	(if (eqv? (cdr list) '())
	    (car list)
	    (tailed-conv (car list) (cdr list))))))

